package com.gxzc.zen.api.sys.service.impl

import com.gxzc.zen.api.sys.model.SysResource
import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.service.ISysPermissionService
import com.gxzc.zen.api.sys.service.ISysResourceService
import com.gxzc.zen.api.sys.service.ISysRoleService
import com.gxzc.zen.common.contants.ZenConstants
import com.gxzc.zen.common.util.RedisCacheUtil
import com.gxzc.zen.umps.common.ZenPermission
import com.gxzc.zen.umps.constant.ZenHttpSession
import com.gxzc.zen.umps.util.SSOUtil
import com.gxzc.zen.umps.util.ShiroRedisUtil
import org.apache.shiro.session.Session
import org.springframework.beans.BeanUtils
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * <p>
 * 权限 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
@Suppress("UNCHECKED_CAST")
class SysPermissionServiceImpl : ISysPermissionService {
    @Autowired
    private lateinit var sysResourceService: ISysResourceService

    @Autowired
    private lateinit var roleService: ISysRoleService

    override fun getAllSysPermission(): MutableList<ZenPermission> {
        var result = RedisCacheUtil.get(ZenConstants.CACHE_KEY_SYS, "allPerms")?.get() as? MutableList<ZenPermission>
        if (result == null || result.isEmpty()) {
            result = mutableListOf()
            val res = sysResourceService.selectList(null)
            res?.forEach {
                result.add(transResource2ZenPermission(it))
            }
        }

//        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
        if (result.isNotEmpty()) {
            RedisCacheUtil.put(ZenConstants.CACHE_KEY_SYS, "allPerms", result)
//            }
        }

        return result
    }

    override fun getSubjectPermission(): MutableList<ZenPermission> {
        val session = SSOUtil.getSession() ?: return mutableListOf()
        var result = session.getAttribute(ZenHttpSession.SESSION_KEY_USER_PERMS) as? MutableList<ZenPermission>
        if (result == null || result.isEmpty()) {
            result = mutableListOf()
            val roles = getSubjectRoles(session)
            val roleIds = roles.map { it.id!! }

            val resources = sysResourceService.getResourcesByRoleIds(roleIds)
            resources.forEach { result.add(transResource2ZenPermission(it)) }
        }
        if (result.isNotEmpty()) {
            session.setAttribute(ZenHttpSession.SESSION_KEY_USER_PERMS, result)
        }

        return result
    }

    override fun getSubjectRoles(session: Session): MutableList<SysRole> {
        var roles = session.getAttribute(ZenHttpSession.SESSION_KEY_USER_ROLES) as? MutableList<SysRole>
        // session中没有怎么办?
        if (roles == null || roles.isEmpty()) {
            // DB中取出来
            roles = roleService.getRoleListByUserId(session.getAttribute(ZenHttpSession.SESSION_KEY_USER_ID) as? Long)
            session.setAttribute(ZenHttpSession.SESSION_KEY_USER_ROLES, roles)
        }
        return roles
    }

    fun transResource2ZenPermission(source: SysResource): ZenPermission {
        val temp = ZenPermission()
        BeanUtils.copyProperties(source, temp)
        if (!(temp.uri != null && temp.uri!!.startsWith("/"))) {
            temp.uri = "/${temp.uri}"
        }
        return temp
    }

    override fun updateAllSysPermission() {
        RedisCacheUtil.evict(ZenConstants.CACHE_KEY_SYS, "allPerms")
    }

    override fun updateAllSessionPermission() {
        ShiroRedisUtil.removeAllSessionsAttributeKey(ZenHttpSession.SESSION_KEY_USER_PERMS)
    }
}
